searchNodes=[{"type":"module","title":"Resource","doc":"An abstraction that provides a way to manage resources in a safe and deterministic manner. Inspired by Bracket monad, it is used to ensure that resources are properly acquired and released, even in the presence of exceptions or other errors. It abstracts three phases: acquire : Takes no arguments and returns a value (acquired resource). use : Accepts a value of acquired resource type and runs computation on it. release : Accepts a value of acquired resource type and runs a &quot;releasing procedure&quot; on it. The key feature of it, is that once acquire succeeded, release is guaranteed to be called under the hood (right after use phase, no matter what it returns or even diverges). So the abstraction ensures that the resource is properly acquired and released, even in the presence of exceptions or other errors. Elixir's kernel already has similar facility — Stream.resource/3 . However, it is biased toward resources of &quot;stream-nature&quot; (working with singleton streams is cumbersome and requires a lot of ceremonies) and does not have a facility to compose resources together (e.g. when you have to perform computation on two acquired at the same time resources). So the main goal of the library is to provide non-biased and composable facility to work with resources in a safe manner. To achieve the first goal the library provide a wrapper on top of Stream.resource/3 . To achieve the second one it utilizes Bindable.ForComprehension . To do so it provides both Bindable.FlatMap and Bindable.Pure implementations for Resource out of the box. So plug in the library, and you get a way to safely combine resources using for-comprehension: iex&gt; import ExUnit.CaptureIO , only : [ capture_io : 1 ] ...&gt; import Bindable.ForComprehension , only : [ bindable : 1 ] ...&gt; import Resource , only : [ create : 1 , use! : 2 ] ...&gt; ...&gt; summation = ...&gt; bindable for x &lt;- create ( acquire : fn -&gt; IO . puts ( &quot;Acquire x&quot; ) ; 1 end , release : fn _ -&gt; IO . puts ( &quot;Release x&quot; ) end ) , ...&gt; y &lt;- create ( acquire : fn -&gt; IO . puts ( &quot;Acquire y&quot; ) ; 2 end , release : fn _ -&gt; IO . puts ( &quot;Release y&quot; ) end ) , ...&gt; do : x + y ...&gt; ...&gt; capture_io fn -&gt; ...&gt; assert_raise RuntimeError , &quot;Boom&quot; , fn -&gt; ...&gt; use! ( summation , fn _sum -&gt; raise &quot;Boom&quot; end ) ...&gt; end ...&gt; end &quot;Acquire x \\n Acquire y \\n Release y \\n Release x \\n &quot;","ref":"Resource.html"},{"type":"function","title":"Resource.create/1","doc":"Resource &quot;data type constructor&quot;.","ref":"Resource.html#create/1"},{"type":"function","title":"Resource.use!/2","doc":"Eagerly runs the provided computation against the resource. If the computation raises, it also raises, but resource is properly released.","ref":"Resource.html#use!/2"},{"type":"type","title":"Resource.t/1","doc":"Resource data type. Though :singleton_stream has to be treated as an implementation details, it was selected in favour to :acquire and :release functions, as it would be more secure in a some sense , due to Elixir does not have &quot;private properties&quot;, so in case of explicit :acquire and :release struct properties, one can access :acquire function and &quot;forget&quot; to run :release .","ref":"Resource.html#t:t/1"},{"type":"extras","title":"Resource","doc":"An abstraction that provides a way to manage resources in a safe and deterministic manner. Inspired by Bracket monad, it is used to ensure that resources are properly acquired and released, even in the presence of exceptions or other errors. It abstracts three phases: acquire : Takes no arguments and returns a value (acquired resource). use : Accepts a value of acquired resource type and runs computation atop of it. release : Accepts a value of acquired resource type and runs a &quot;releasing procedure&quot; on it. The key feature of it, is that once acquire succeeded, release is guaranteed to be called under the hood (right after use phase, no matter what it returns or even diverges). So the abstraction ensures that the resource is properly acquired and released, even in the presence of exceptions or other errors. Elixir's kernel already has similar facility — Stream.resource/3 . However, it is biased toward resources of &quot;stream-nature&quot; and does not have a facility to compose resources together (e.g. when you have to perform computation on two acquired resources). So the main goal of the library is to provide non-biased and composable facility to work with resources in a safe manner. To achieve the first goal the library provide a wrapper on top of Stream.resource/3 . To achieve the second one it utilizes Bindable.ForComprehension . To do so it provides both Bindable.FlatMap and Bindable.Pure implementations for Resource out of the box. So plug in the library, and you get a way to safely combine resources using for-comprehension: import Bindable.ForComprehension import Resource summation = bindable for x &lt;- create ( acquire : fn -&gt; IO . puts ( &quot;Acquire x&quot; ) ; 40 end , release : fn _ -&gt; IO . puts ( &quot;Release x&quot; ) end ) , y &lt;- create ( acquire : fn -&gt; IO . puts ( &quot;Acquire y&quot; ) ; 2 end , release : fn _ -&gt; IO . puts ( &quot;Release y&quot; ) end ) , do : x + y use! ( summation , fn sum -&gt; IO . puts ( &quot;Use sum&quot; ) ; { sum } end )","ref":"readme.html"},{"type":"extras","title":"Resource - Installation","doc":"The package can be installed by adding resource to your list of dependencies in mix.exs : def deps do [ { :resource , &quot;~&gt; 1.0.0&quot; } ] end The docs can be found at https://hexdocs.pm/resource .","ref":"readme.html#installation"}]